%
% Graphs.m
%
classdef Graphs < handle

  methods (Static)
    
    %
    % 観測値とその推定値 y の時間推移グラフを生成する。
    % y が多次元の場合には、次元毎に複数グラフを生成する。
    % 
    % YE(outputFileNamePrefix, matFileName, iterations)
    % YE(outputFileNamePrefix, matFileName, iterations, times)
    %   outputFileNamePrefix: 出力グラフファイル (PDF) のファイル名接頭辞
    %   matFileName: グラフ出力対象データの .mat ファイル名
    %   iterations: プロットする推定値の Iteration 配列
    %   times: グラフ出力する Time 範囲。省略時はすべての Time を出力
    %
    function YE(outputFileNamePrefix, matFileName, iterations, varargin)
      load(matFileName);
      % in.z, out.j, out.YE loaded
      
      J = out.j;
      T = size(out.YE, 2);
      dim = size(out.YE, 3);
      if (nargin >= 4)
        orgTs = varargin{1};
        ts = orgTs(1 <= orgTs & orgTs <= T);
      else
        ts = 1:T;
      end
      
      for d = 1:dim
        g = PlotGraph();
        g.Title('YE');
        g.XLabel('Time');
        g.YLabel('Y');
        % z
        g.AddSeries('z', ts, in.z(d, ts));
        % YE
        for i = iterations
          if (1 <= i && i <= J)
            ye = out.YEAt(i);
            g.AddSeries(num2str(i), ts, ye(1, ts, d));
          end
        end
        
        if (dim == 1)
          g.SaveAs(Graphs.pdfFileName(outputFileNamePrefix));
        else
          g.SaveAs(Graphs.pdfNumberedFileName(outputFileNamePrefix, d));
        end
      end
    end
    
    %
    % 状態変数 x の時間推移グラフを生成する。
    % x が多次元の場合には、次元毎に複数グラフを生成する。
    % 
    % XE(outputFileNamePrefix, matFileName, iterations)
    % XE(outputFileNamePrefix, matFileName, iterations, times)
    %   outputFileNamePrefix: 出力グラフファイル (PDF) のファイル名接頭辞
    %   matFileName: グラフ出力対象データの .mat ファイル名
    %   iterations: プロットする状態変数の Iteration 配列
    %   times: グラフ出力する Time 範囲。省略時はすべての Time を出力
    %
    function XE(outputFileNamePrefix, matFileName, iterations, varargin)
      load(matFileName);
      % out.j, out.XE loaded.
      
      J = out.j;
      T = size(out.XE, 2);
      dim = size(out.XE, 3);
      if (nargin >= 4)
        orgTs = varargin{1};
        ts = orgTs(1 <= orgTs & orgTs <= T);
      else
        ts = 1:T;
      end
      
      for d = 1:dim
        g = PlotGraph();
        g.Title('XE');
        g.XLabel('Time');
        g.YLabel('X');
        for i = iterations
          if (1 <= i && i <= J)
            xe = out.XEAt(i);
            g.AddSeries(num2str(i), ts, xe(1, ts, d));
          end
        end
        
        if (dim == 1)
          g.SaveAs(Graphs.pdfFileName(outputFileNamePrefix));
        else
          g.SaveAs(Graphs.pdfNumberedFileName(outputFileNamePrefix, d));
        end
      end
    end

    %
    % 観測値とその推定平均 y の Iteration 推移グラフを生成する。
    % y が多次元の場合には、次元毎に複数グラフを生成する。
    % 
    % YEMean(outputFileNamePrefix, matFileName)
    % YEMean(outputFileNamePrefix, matFileName, times)
    %   outputFileNamePrefix: 出力グラフファイル (PDF) のファイル名接頭辞
    %   matFileName: グラフ出力対象データの .mat ファイル名
    %   times: グラフ出力する Time 範囲。省略時はすべての Time を出力
    %
    function YEMean(outputFileNamePrefix, matFileName, varargin)
      load(matFileName);
      % in.z, out.j, out.YE loaded.

      J = out.j;
      T = size(out.YE, 2);
      dim = size(out.YE, 3);
      if (nargin >= 3)
        orgTs = varargin{1};
        ts = orgTs(1 <= orgTs & orgTs <= T);
      else
        ts = 1:T;
      end

      YE = out.YEAt(1:J);
      YEMean = squeeze(mean(YE(:, ts, :), 1));
      YEStd = squeeze(std(YE(:, ts, :), 1));

      for d = 1:dim
        g = PlotGraph();
        g.Title('YE_Mean');
        g.XLabel('Time');
        g.YLabel('Y');
        % z
        g.AddSeries('z', ts, in.z(d, ts), 'Color', 'red');
        % YE_Mean
        g.AddSeries('Mean', ts, YEMean(:, d), 'Color', 'blue');
        g.AddSeries('+sigma', ts, YEMean(:, d) + YEStd(:, d), 'Color', 'blue', 'LineStyle', '--');
        g.AddSeries('-sigma', ts, YEMean(:, d) - YEStd(:, d), 'Color', 'blue', 'LineStyle', '--');
        if (dim == 1)
          g.SaveAs(Graphs.pdfFileName(outputFileNamePrefix));
        else
          g.SaveAs(Graphs.pdfNumberedFileName(outputFileNamePrefix, d));
        end
      end
    end

    %
    % 状態変数 x の推定平均の Iteration 推移グラフを生成する。
    % x が多次元の場合には、次元毎に複数グラフを生成する。
    % 
    % XEMean(outputFileNamePrefix, x, matFileName)
    % XEMean(outputFileNamePrefix, x, matFileName, times)
    %   outputFileNamePrefix: 出力グラフファイル (PDF) のファイル名接頭辞
    %   matFileName: グラフ出力対象データの .mat ファイル名
    %   times: グラフ出力する Time 範囲。省略時はすべての Time を出力
    %
    function XEMean(outputFileNamePrefix, matFileName, varargin)
      load(matFileName);
      % out.j, out.XE loaded.

      J = out.j;
      T = size(out.XE, 2);
      dim = size(out.XE, 3);
      if (nargin >= 3)
        orgTs = varargin{1};
        ts = orgTs(1 <= orgTs & orgTs <= T);
      else
        ts = 1:T;
      end

      XE = out.XEAt(1:J);
      XEMean = squeeze(mean(XE(:, ts, :), 1));
      XEStd = squeeze(std(XE(:, ts, :), 1));

      for d = 1:dim
        g = PlotGraph();
        g.Title('XE_Mean');
        g.XLabel('Time');
        g.YLabel('X');
        % XE_Mean
        g.AddSeries('Mean', ts, XEMean(:, d), 'Color', 'blue');
        g.AddSeries('+sigma', ts, XEMean(:, d) + XEStd(:, d), 'Color', 'blue', 'LineStyle', '--');
        g.AddSeries('-sigma', ts, XEMean(:, d) - XEStd(:, d), 'Color', 'blue', 'LineStyle', '--');
        if (dim == 1)
          g.SaveAs(Graphs.pdfFileName(outputFileNamePrefix));
        else
          g.SaveAs(Graphs.pdfNumberedFileName(outputFileNamePrefix, d));
        end
      end
    end

    %
    % 尤度の Iteration 推移グラフを生成する。
    % 
    % Loglik = Loglik(outputFileName, matFileName1, matFileName2, ...)
    % パラメータ
    %   outputFileName: 出力グラフファイル (PDF) 名 
    %   matFileName1, matFileName2, ...: .mat ファイル名（複数可）
    % 戻り値
    %   Loglik: 尤度
    %
    function Loglik = Loglik(outputFileName, varargin)
      Loglik = Graphs.CalcLoglik(varargin{:});

      g = PlotGraph();
      g.Title('Log likelihood');
      g.XLabel('Iteration');
      g.YLabel('Log likelihood');
      for i = 1:length(Loglik)
        matFileName = varargin{i};
        x = 1:length(Loglik{i});
        y = Loglik{i};
        g.AddSeries(matFileName, x, y);
      end
      g.SaveAs(Graphs.pdfFileName(outputFileName));
    end

    %
    % 尤度の Iteration 推移データを出力する。
    % 
    % Loglik = CalcLoglik(matFileName1, matFileName2, ...)
    % パラメータ
    %   matFileName1, matFileName2, ...: .mat ファイル名（複数可）
    % 戻り値
    %   Loglik: 尤度
    %
    function Loglik = CalcLoglik(varargin)
      if (nargin < 1)
        error('Specify matFileName(s).');
      end

      Loglik = {};
      for mfn = varargin
        matFileName = mfn{1};
	load(matFileName);
	% out.j, out.loglik loaded.
	Loglik{length(Loglik) + 1} = out.loglikAt(1:out.j)';
      end
    end

    %
    % 平均二乗誤差の Iteration 推移グラフを生成する。
    % 
    % [RmseMean, RmseStd, Rmse] = Rmse(matFileName)
    % パラメータ
    %   outputFileName: 出力グラフファイル (PDF) 名
    %   matFileName: グラフ出力対象データの .mat ファイル名
    % 戻り値
    %   RmseMean: RMSE の平均
    %   RmseStd:  RMSE の標準偏差
    %   Rmse:     RMSE
    %
    function [RmseMean, RmseStd, Rmse] = Rmse(outputFileName, matFileName)
      [RmseMean, RmseStd, Rmse] = Graphs.CalcRmse(matFileName);

      J = length(RmseMean);

      g = PlotGraph();
      g.Title('RMSE');
      g.XLabel('Iteration');
      g.YLabel('RMSE');
      g.AddSeries('Mean', 1:J, RmseMean, 'Color', 'blue');
      g.AddSeries('+sigma', 1:J, RmseMean + RmseStd, 'Color', 'blue', 'LineStyle', '--');
      g.AddSeries('-sigma', 1:J, RmseMean - RmseStd, 'Color', 'blue', 'LineStyle', '--');
      g.SaveAs(Graphs.pdfFileName(outputFileName));
    end

    %
    % 平均二乗誤差の Iteration 推移データを計算する。
    %
    % [RmseMean, RmseStd, Rmse] = CalcRmse(matFileName)
    % パラメータ
    %   matFileName: .mat ファイル名
    % 戻り値
    %   RmseMean: RMSE の平均
    %   RmseStd:  RMSE の標準偏差
    %   Rmse:     RMSE
    %
    function [RmseMean, RmseStd, Rmse] = CalcRmse(matFileName)
      load(matFileName);
      % in.z, out.j, out.YE loaded.

      J = out.j;
      T = size(out.YE, 2);
      dim = size(out.YE, 3);

      YE_mean = cumsum(out.YEAt(1:J), 1) ./ repmat([1:J]', 1, T, dim);
      Rmse = sqrt(sum((YE_mean - permute(repmat(in.z', 1, 1, J), [3 1 2])) .^ 2, 3));
      RmseMean = mean(Rmse, 2);
      RmseStd = std(Rmse, 0, 2);
    end    

    %
    % 推定した状態値関数 f のグラフを生成する。
    % 状態値が多次元の場合には、値を変化させるひとつの次元を除き値を固定とする。
    % また、グラフは状態値の次元毎に生成する。
    % 
    % FV(outputFileNamePrefix, matFileName, iterations, x1, x2, x3, ...)
    %   outputFileNamePrefix: 出力グラフファイル (PDF) のファイル名接頭辞
    %   matFileName: グラフ出力対象データの .mat ファイル名
    %   iterations: プロットする Iteration 配列
    %   x1, x2, x3, ...: 次元毎の固定値、もしくは値範囲指定
    % 
    % 例:
    % 状態値が 3 次元で、Iteration 1, 10, 100 の状態値関数を、
    % 1 次元目の値範囲 -1:.1:1, 2 次元目 0, 3 次元目 0 のグラフを生成する。
    % >> Graphs.FV('test', 'test.mat', [1 10 100], -1:.1:1, 0, 0)
    % 
    function FV(outputFileNamePrefix, ...
                matFileName, ...
                iterations, ...
                varargin)
      load(matFileName);
      % in.splineHandle, in.xGrids, out.j, out.FV loaded.
      Graphs.fvOrGvGraph('FV', 'X(t+1)', outputFileNamePrefix, in.splineHandle, iterations, in.xGrids, @(i) out.FVAt(i), out.j, varargin{:});
    end
    
    %
    % 推定した観測値関数 g のグラフを生成する。
    % 状態値が多次元の場合には、値を変化させるひとつの次元を除き値を固定とする。
    % また、グラフは観測値の次元毎に生成する。
    % 
    % GV(outputFileNamePrefix, matFileName, iterations, x1, x2, x3, ...)
    %   outputFileNamePrefix: 出力グラフファイル (PDF) のファイル名接頭辞
    %   matFileName: グラフ出力対象データの .mat ファイル名
    %   iterations: プロットする Iteration 配列
    %   x1, x2, x3, ...: 次元毎の固定値、もしくは値範囲指定
    % 
    % 例:
    % 状態値が 3 次元で、Iteration 1, 10, 100 の観測値関数を、
    % 1 次元目の値範囲 -1:.1:1, 2 次元目 0, 3 次元目 0 のグラフを生成する。
    % >> Graphs.GV('test', 'test.mat', [1 10 100], -1:.1:1, 0, 0)
    % 
    function GV(outputFileNamePrefix, ...
                matFileName, ...
                iterations, ...
                varargin)
      load(matFileName);
      % in.splineHandle, in.xGrids, out.j, out.GV loaded.
      Graphs.fvOrGvGraph('GV', 'Y(t)', outputFileNamePrefix, in.splineHandle, iterations, in.xGrids, @(i) out.GVAt(i), out.j, varargin{:});
    end
    
    %
    % 状態値カーネル関数パラメータの、Iteration 推移グラフを生成する。
    % パラメータ毎に複数グラフを生成する。
    %   outputFileNamePrefix: 出力グラフファイル (PDF) のファイル名接頭辞
    %   matFileName: グラフ出力対象データの .mat ファイル名
    %
    function SKP(outputFileNamePrefix, matFileName)
      load(matFileName);
      % out.j, out.SKP loaded.
      Graphs.kpGraph('SKP', outputFileNamePrefix, out.SKPAt(1:out.j));
    end
    
    %
    % 観測値カーネル関数パラメータの、Iteration 推移グラフを生成する。
    % パラメータ毎に複数グラフを生成する。
    %   outputFileNamePrefix: 出力グラフファイル (PDF) のファイル名接頭辞
    %   matFileName: グラフ出力対象データの .mat ファイル名
    %
    function OKP(outputFileNamePrefix, matFileName)
      load(matFileName);
      % out.j, out.OKP loaded.
      Graphs.kpGraph('OKP', outputFileNamePrefix, out.OKPAt(1:out.j));
    end
    
  end
  
  methods (Static, Access = private)
    
    function result = pdfFileName(str)
      tokens = regexpi(str, '(.*)\.pdf$', 'tokens');
      if (size(tokens, 2) == 1)
        result = str;
      else
        result = sprintf('%s.pdf', str);
      end
    end
    
    function result = pdfNumberedFileName(str, i)
      tokens = regexpi(str, '(.*)\.pdf$', 'tokens');
      if (size(tokens, 2) == 1)
        result = sprintf('%s_%d.pdf', tokens{1}{1}, i);
      else
        result = sprintf('%s_%d.pdf', str, i);
      end
    end
    
    function fvOrGvGraph(title, ...
                         ylabel, ...
                         outputFileNamePrefix, ...
                         splineHandle, ...
                         iterations, ...
                         xGrids, ...
                         eqnValsAt, ...
                         J, ...
                         varargin)
      inDim = size(xGrids, 2);
      varargSize = nargin - 8;
      
      % varargin check
      if (varargSize ~= inDim)
        error(sprintf('Needs args for %d dimension(s).', inDim));
      end
      rangeDim = 0;
      rangeSize = 0;
      for i = 1:varargSize
        s = size(varargin{i}, 2);
        if (s > 1)
          if (rangeDim ~= 0)
            error('Not allowed to specify a range to multi dimensions.');
          end
          rangeDim = i;
          rangeSize = s;
        end
      end
      if (rangeDim == 0)
        error('Specify a range to a single dimension.');
      end
      graphX = varargin{rangeDim};
      funcX = [];
      for i = 1:varargSize
        if (i == rangeDim)
          funcX = vertcat(funcX, varargin{i});
        else
          funcX = vertcat(funcX, repmat(varargin{i}, 1, rangeSize));
        end
      end
      
      % function execute.
      validIterations = [];
      funcResults = [];
      gridData = GridData(xGrids);
      for i = iterations
        if (1 <= i && i <= J)
          validIterations = horzcat(validIterations, i);
          functions = ModelFunctions(splineHandle, gridData, squeeze(eqnValsAt(i)));
          f = VectorValuedFunction(functions);
          funcResults = cat(3, funcResults, f(funcX'));
        end
      end
      
      % Graph
      outDim = size(funcResults, 2);
      for d = 1:outDim
        g = PlotGraph();
        g.Title(title);
        g.XLabel('X(t)');
        g.YLabel(ylabel);
        for i = 1:size(validIterations, 2)
          g.AddSeries(num2str(validIterations(i)), graphX, funcResults(:, d, i)');
        end
        
        if (outDim == 1)
          g.SaveAs(Graphs.pdfFileName(outputFileNamePrefix));
        else
          g.SaveAs(Graphs.pdfNumberedFileName(outputFileNamePrefix, d));
        end
      end
      
    end
    
    function kpGraph(title, outputFileNamePrefix, kp)
      J = size(kp, 1);
      dim = size(kp, 2);
      pc = size(kp, 3);
      
      pnames = { 'Sigma', 'L' };
      for p = 1:2
        g = PlotGraph();
        g.Title(title);
        g.XLabel('Iteration');
        g.YLabel(pnames{p});
        for d = 1:dim
          g.AddSeries(sprintf('dim %d', d), 1:J, kp(1:J, d, p));
        end
        g.SaveAs(sprintf('%s_%s', outputFileNamePrefix, pnames{p}));
      end
      
    end
    
  end
  
end
