%
% ParticleFilter.m
%

classdef ParticleFilter < handle

  properties (Access = protected)
    ancestorIndicies
  end
  
  properties (SetAccess = protected)
    Particles
    Weights
    Loglik
  end
  
  methods (Access = public)
  
    % *** GWP: NOTE - this is for an SIR filter - i.e sample for each particle
    % at time t for index i from prior for state equation
    % weight by the observation equation (i.e. likelihood)
    
    % Particle filter
    % Input:
    %   x0 - initial condition (1 * xdim)
    %   xaux - auxiliary condigion (T * adim)
    %   u  - control (T * xdim)
    %   y  - measurements (T * ydim)
    %   f  - state transition function (R^xdim -> R^xdim)
    %   h  - observation function (R^xdim -> R^ydim)
    %   q  - process noise variance
    %   r  - measurement noise variance
    %   N  - number of particles
    %   K  - observation offset
    function this = ParticleFilter(x0, xaux, u, y, f, h, q, r, N, K)
      % Retrieve dimensions from input data
      xdim = size(x0, 2);
      ydim = size(y, 2);
      T = size(y, 1);
      
      % Initialization
      this.Particles = zeros(N, T, xdim);
      this.ancestorIndicies = repmat([1:N]', 1, T);
      this.Weights = zeros(N, T);
      
      this.Particles(:, 1, :) = repmat(x0, N, 1);
      this.Loglik = 0;
      
      for t = (K + 1):T
      
        if t ~= K + 1
          ind = ParticleFilter.resampling(this.Weights(:, t - K - 1));
          if length(xaux) > 0
            xpred = f([squeeze(this.Particles(:, t - K - 1, :)) repmat(xaux(t - K - 1, :), N, 1)]);
          else
            xpred = f(squeeze(this.Particles(:, t - K - 1, :)));
          end
          if length(u) > 0
            xpred = xpred + repmat(u(t - K - 1, :), N, 1);
          end
          % TODO: x が多次元になったときの process noise (q) の加え方がこれで正しいか確認する
          this.Particles(:, t - K, :) = xpred(ind, :) + sqrt(q) * randn(N, xdim);
          this.ancestorIndicies(:, t - K) = ind;
        end
        
        % Compute importance weights
        % TODO: y が多次元になったときの logweights の計算がこれで正しいか確認する
        if length(xaux) > 0
          ypred = h([squeeze(this.Particles(:, t - K, :)) repmat(xaux(t - K, :), N, 1)]);
        else
          ypred = h(squeeze(this.Particles(:, t - K, :)));
        end
        logweights = -1 / 2 * log(2 * pi * r) - 1 / (2 * r) * sum((repmat(y(t, :), N, 1) - ypred) .^ 2, 2);
        const = max(logweights);
        weights = exp(logweights - const);
        
        % Compute loglikelihood
        this.Loglik = this.Loglik + const + log(sum(weights)) - log(N);
        this.Weights(:, t - K) = weights / sum(weights);
        
      end
    end
    
  end
  
  methods (Static, Access = private)
    
    % ======================================================================
    %
    %   Multinomial resampling function
    %
    % ======================================================================
    function i = resampling(q)
      N = length(q);
      % Multinomial resampling
      u = rand(N, 1);
      qc = cumsum(q);
      qc = qc(:);
      qc = qc / qc(N);
      [~, ind1] = sort([u; qc]);
      ind2 = find(ind1 <= N);
      i = ind2' - (0:N-1);
    end
    
  end
  
end
